﻿/*
********************************************************************************
* COPYRIGHT(c) ЗАО «ЧИП и ДИП», 2019
* 
* Программное обеспечение предоставляется на условиях «как есть» (as is).
* При распространении указание автора обязательно.
********************************************************************************
*/


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace SpectrA
{
    public class Column : INotifyPropertyChanged
    {   
        private static readonly string NULL_BACKGROUND_COLOR = "#E3E6E3";

        private UInt16 startfreqIndex = 1;
        private UInt16 endfreqIndex = 1;
        private string multiplier = "0,0001";
        private string freqrange = " ";

        public List<Pixel> Pixels { get; set; }        
        public byte LastActivePixel { get; set; }
        public byte PeakPixel { get; set; }


        public Column(byte PixelInColumn)
        {
            Pixels = new List<Pixel>();

            for (byte i = 0; i < PixelInColumn; i++)
            {
                Pixel NewPixel = new Pixel();
                Pixels.Add(NewPixel);
            }

            SetFreqRangeValue();
        }

        public UInt16 FreqIndexStart
        {
            get { return this.startfreqIndex; }
            set
            {
                if (this.startfreqIndex != value)
                {
                    this.startfreqIndex = value;

                    SetFreqRangeValue();

                    this.NotifyPropertyChanged("StartFreqIndex");
                }
            }
        }

        public ushort FreqIndexEnd
        {
            get { return this.endfreqIndex; }
            set
            {
                if (this.endfreqIndex != value)
                {
                    this.endfreqIndex = value;

                    SetFreqRangeValue();

                    this.NotifyPropertyChanged("EndFreqIndex");
                }
            }
        }

        public string Multiplier
        {
            get { return this.multiplier; }
            set
            {
                if (this.multiplier != value)
                {
                    this.multiplier = value;
                    this.NotifyPropertyChanged("Multiplier");
                }
            }
        }

        public string FreqRange
        {
            get { return this.freqrange; }
            set
            {
                if (this.freqrange != value)
                {
                    this.freqrange = value;
                    this.NotifyPropertyChanged("FreqRange");
                }
            }
        }
        
        public void Update(MatrixColors Colors, MatrixOptions Options)
        {            
            for (int PixelNum = 0; PixelNum < Pixels.Count; PixelNum++)
            {
                if (PixelNum < LastActivePixel)
                {
                    if (PixelNum < Options.MiddleZoneStart)
                        Pixels[PixelNum].Color = Colors.LowZone;
                    else if (PixelNum <= Options.MiddleZoneEnd)
                        Pixels[PixelNum].Color = Colors.MiddleZone;
                    else
                        Pixels[PixelNum].Color = Colors.HighZone;
                }

                else
                {
                    if (Options.IsBackgroundEnabled == true)
                        Pixels[PixelNum].Color = Colors.Background;
                    else
                        Pixels[PixelNum].Color = NULL_BACKGROUND_COLOR;
                }
            }

            Pixels[PeakPixel].Color = Colors.Peak;            
        }

        public void Brush(string Color)
        {
            for (int PixelNum = 0; PixelNum < Pixels.Count; PixelNum++)
                Pixels[PixelNum].Color = Color;
        }
        
        public event PropertyChangedEventHandler PropertyChanged;
        public void NotifyPropertyChanged(string propName)
        {
            this.PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propName));
        }        

        private void SetFreqRangeValue()
        {
            string FreqStartString = ((startfreqIndex + 1) * MainWindow.FREQ_STEP).ToString("f0");
            
            if (startfreqIndex == endfreqIndex)
                FreqRange = FreqStartString + " Гц";
            else
            {
                string FreqEndString = ((endfreqIndex + 1) * MainWindow.FREQ_STEP).ToString("f0");

                if (startfreqIndex > endfreqIndex)
                {
                    string SwapString = FreqEndString;
                    FreqEndString = FreqStartString;
                    FreqStartString = SwapString;
                }

                FreqRange = FreqStartString + "-" + FreqEndString + " Гц";
            }            
        }
    }
}
