﻿/*
********************************************************************************
* COPYRIGHT(c) ЗАО «ЧИП и ДИП», 2019
* 
* Программное обеспечение предоставляется на условиях «как есть» (as is).
* При распространении указание автора обязательно.
********************************************************************************
*/


using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace SpectrA
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        const UInt32 USB_PACKET_SIZE = 64;
        const byte USB_REPORT_ID = 1;

        const byte USB_CMD_POS = 0x01;
        const byte USB_SUBCMD_POS = 0x02;
        const byte USB_COLUMN_POS = 0x03;
        const byte USB_DATA_POS = 0x04;

        const byte USB_CMD_WRITE = 0;
        const byte USB_CMD_READ = 1;

        const byte USB_SUBCMD_FIRMWARE = 0;
        const byte USB_SUBCMD_MATRIX_SET = 1;
        const byte USB_SUBCMD_COLUMN_SET = 2;

        const byte MATRIX_LOW_COLOR_SIZE = 3;
        const byte MATRIX_MIDDLE_COLOR_SIZE = MATRIX_LOW_COLOR_SIZE;
        const byte MATRIX_HIGH_COLOR_SIZE = MATRIX_LOW_COLOR_SIZE;
        const byte MATRIX_PEAK_COLOR_SIZE = MATRIX_LOW_COLOR_SIZE;
        const byte MATRIX_BACKGROUND_SIZE = MATRIX_LOW_COLOR_SIZE;
        const byte MATRIX_FRAME_COLOR_SIZE = MATRIX_LOW_COLOR_SIZE;
        const byte MATRIX_MIDDLE_ZONE_START_SIZE = 1;
        const byte MATRIX_MIDDLE_ZONE_END_SIZE = MATRIX_MIDDLE_ZONE_START_SIZE;
        const byte MATRIX_SHOW_OPTIONS_SIZE = 1;

        const byte MATRIX_LOW_COLOR_OFFSET = 0;
        const byte MATRIX_MIDDLE_COLOR_OFFSET = (MATRIX_LOW_COLOR_OFFSET + MATRIX_LOW_COLOR_SIZE);
        const byte MATRIX_HIGH_COLOR_OFFSET = (MATRIX_MIDDLE_COLOR_OFFSET + MATRIX_MIDDLE_COLOR_SIZE);
        const byte MATRIX_PEAK_COLOR_OFFSET = (MATRIX_HIGH_COLOR_OFFSET + MATRIX_HIGH_COLOR_SIZE);
        const byte MATRIX_BACKGROUND_OFFSET = (MATRIX_PEAK_COLOR_OFFSET + MATRIX_PEAK_COLOR_SIZE);
        const byte MATRIX_FRAME_COLOR_OFFSET = (MATRIX_BACKGROUND_OFFSET + MATRIX_BACKGROUND_SIZE);
        const byte MATRIX_MIDDLE_ZONE_START_OFFSET = (MATRIX_FRAME_COLOR_OFFSET + MATRIX_FRAME_COLOR_SIZE);
        const byte MATRIX_MIDDLE_ZONE_END_OFFSET = (MATRIX_MIDDLE_ZONE_START_OFFSET + MATRIX_MIDDLE_ZONE_START_SIZE);
        const byte MATRIX_SHOW_OPTIONS_OFFSET = (MATRIX_MIDDLE_ZONE_END_OFFSET + MATRIX_MIDDLE_ZONE_END_SIZE);

        const byte COLUMN_FREQ_START_SIZE = 2;
        const byte COLUMN_FREQ_END_SIZE = COLUMN_FREQ_START_SIZE;
        const byte COLUMN_MULTIPLIER_SIZE = 4;
        const byte COLUMN_SETTINGS_SIZE = (COLUMN_FREQ_START_SIZE + COLUMN_FREQ_END_SIZE + COLUMN_MULTIPLIER_SIZE);

        const byte COLUMN_FREQ_START_OFFSET = 0;
        const byte COLUMN_FREQ_END_OFFSET = (COLUMN_FREQ_START_OFFSET + COLUMN_FREQ_START_SIZE);
        const byte COLUMN_MULTIPLIER_OFFSET = (COLUMN_FREQ_END_OFFSET + COLUMN_FREQ_END_SIZE);
        
        const byte COLUMNS_PER_EEPROM_PAGE = 4;
        
        const byte BACKGROUND_OPTION = (1 << 0);
        const byte FRAME_OPTION = (1 << 1);
        const byte OPTIONS_MASK = BACKGROUND_OPTION | FRAME_OPTION;

        const byte ColorStringLength = 7;

        const byte MATRIX_COLUMNS = 18;
        const byte MATRIX_ROWS = 14;
        public const float FREQ_STEP = 39.0625F;
        const UInt16 FREQ_COUNT = 512;
        
        private static readonly string LowZoneColorDef = "#00FF00";
        private static readonly string MiddleZoneColorDef = "#FFFF00";
        private static readonly string HighZoneColorDef = "#FF5D00";
        private static readonly string PeakColorDef = "#FF0000";
        private static readonly string BackgroundDef = "#010000";
        private static readonly string FrameColDef = "#010101";

        const byte MiddleZoneStartDefault = 5;
        const byte MiddleZoneEndDefault = 8;

        const byte MatrixColorsCount = 6;
        const byte MatrixOptionsCount = 4;

        const int SelectThickness = 3;
        const int DeSelectThickness = 1;

        //шаг частот 39,0625 Гц
        private static readonly UInt16[] DefaultBandsStart =
        {
         1,   3,   6,  10,  20,  26,  32,  51,  64, //39Гц, 117Гц, 234Гц, 390Гц, 781Гц, 1015Гц, 1250Гц, 1992Гц, 2500Гц,
        80, 102, 128, 154, 205, 256, 307, 358, 410,//3125Гц, 3984Гц, 5кГц, 6кГц, 8кГц, 10кГц, 12кГц, 14кГц, 16кГц,
  
        };

        private static readonly UInt16[] DefaultBandsEnd =
        {
         1,   3,   6,  10,  20,  26,  32,  51,  64, //39Гц, 117Гц, 234Гц, 390Гц, 781Гц, 1015Гц, 1250Гц, 1992Гц, 2500Гц,
        80, 102, 128, 154, 205, 256, 307, 358, 410,//3125Гц, 3984Гц, 5кГц, 6кГц, 8кГц, 10кГц, 12кГц, 14кГц, 16кГц,
        };


        private static readonly float[] DefaultMultipliers =
        {
        0.0005F, 0.001F, 0.001F, 0.003F, 0.003F, 0.003F, 0.01F, 0.05F, 0.03F,
        0.03F,   0.03F,  0.05F,  0.05F,  0.01F,  0.01F,  0.05F, 0.02F, 0.2F,
        };

        private static readonly string DeviceConnectedString = "Анализатор спектра подключен.";
        private static readonly string DeviceNotConnectedString = "Анализатор спектра не подключен. Подключите устройство и перезапустите программу.";
        private static readonly string FirmwareVersionPhrase = "Версия ПО устройства ";

        private static readonly string UserInputErrorHeader = "Ошибка ввода данных";
        private static readonly string UserInputErrorColor = "Компоненты цвета R, G, B не должны превышать значения " + COLOR_COMPONENT_MAX_VALUE.ToString();

        private static readonly string MatrixSettingsFileString = "Matrix Settings";
        private static readonly string LowZoneFileString = "LowZoneColor";
        private static readonly string MiddleZoneFileString = "MiddleZoneColor";
        private static readonly string HighZoneFileString = "HighZoneColor";
        private static readonly string PeakFileString = "Peak";
        private static readonly string BackgroundFileString = "Background";
        private static readonly string FrameFileString = "Frame";
        private static readonly string MiddleZoneStartFileString = "MiddleZoneStart";
        private static readonly string MiddleZoneEndFileString = "MiddleZoneEnd";
        private static readonly string IsBackgroundFileString = "IsBackgroundEnabled";
        private static readonly string IsFrameFileString = "IsFrameEnabled";
        private static readonly string ColumnsSettingsFileString = "Columns Settings";
        private static readonly string ColumnsFreqStartFileString = "FrequencyStart";
        private static readonly string ColumnsFreqEndFileString = "FrequencyEnd";
        private static readonly string ColumnsMultiplierFileString = "Multiplier";

        private static readonly string[] ColorsFileStrings =
        {
            LowZoneFileString, MiddleZoneFileString, HighZoneFileString,
            PeakFileString,    BackgroundFileString, FrameFileString,
        };

        const byte COLOR_COMPONENT_MAX_VALUE = 5;


        const byte LOW_ZONE_COLOR_INDEX = 0;
        const byte MIDDLE_ZONE_COLOR_INDEX = 1;
        const byte HIGH_ZONE_COLOR_INDEX = 2;
        const byte PEAK_COLOR_INDEX = 3;
        const byte BACKGROUND_INDEX = 4;
        const byte FRAME_COLOR_INDEX = 5;

        const UInt32 UINT_FLOAT_MULTIPLIER = 10000;

        struct ThreadOperation
        {
            public byte Operation;
            public byte Property;
            public byte StartColumn;

            public ThreadOperation(byte operation, byte property, byte startcol)
            {
                Operation = operation;
                Property = property;
                StartColumn = startcol;
            }
        }

        public PixelMatrix Matrix_18x14 { get; set; }        
        public List<string> Frequencies;
        private int ColorIndex = -1;
        private DispatcherTimer ThreadTimer = new DispatcherTimer();
        private ProgressWindow ProgressBar = new ProgressWindow();
        private ThreadOperation DeviceReadWrite = new ThreadOperation();

        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
            
            Matrix_18x14 = new PixelMatrix(MATRIX_COLUMNS, MATRIX_ROWS);
            SetMatrixToDefault();
            
            ColumnList.ItemsSource = Matrix_18x14.Columns;          
            
            Frequencies = new List<string>();
            for (UInt16 FreqNum = 1; FreqNum <= FREQ_COUNT; FreqNum++)
            {
                Frequencies.Add((FreqNum * FREQ_STEP).ToString("f0"));
            }

            StartFreq.ItemsSource = Frequencies;
            EndFreq.ItemsSource = Frequencies;
            
            Boolean USBDevDetected = USB_device.Open();

            ShowConnectionState(USBDevDetected);
            
            if (USBDevDetected)
            {
                byte[] USBPacket = new byte[USB_PACKET_SIZE];
                USBPacket[0] = USB_REPORT_ID;
                USBPacket[USB_CMD_POS] = USB_CMD_READ;
                USBPacket[USB_SUBCMD_POS] = USB_SUBCMD_FIRMWARE;

                Boolean USBSuccess = USB_device.Write(USBPacket);
                USBSuccess = USB_device.Read(USBPacket);
                ShowConnectionState(USBSuccess);

                byte FirmwareVersion = USBPacket[USB_DATA_POS];
                FirmwareTextBlock.Text = FirmwareVersionPhrase + (FirmwareVersion >> 4).ToString() + "." + (FirmwareVersion & 0x0F).ToString();

                DeviceReadWrite.Operation = USB_CMD_READ;
                DeviceReadWrite.Property = USB_SUBCMD_MATRIX_SET;
                DeviceReadWrite.StartColumn = 0;

                ThreadTimer.Interval = TimeSpan.FromMilliseconds(30); //период опроса устройства
                ThreadTimer.Tick += ThreadTimer_Tick;
                ThreadTimer.Start();
            }            
        }
        
        private void Peak_MouseLeftButtonClick(object sender, MouseButtonEventArgs e)
        {
            HighlightBorder(PeakBorder);
            ColorIndex = PEAK_COLOR_INDEX;
            ColorSelect.HexadecimalString = PeakBorder.Background.ToString();
        }

        private void HighZone_MouseLeftButtonClick(object sender, MouseButtonEventArgs e)
        {
            HighlightBorder(HighZoneBorder);
            ColorIndex = HIGH_ZONE_COLOR_INDEX;
            ColorSelect.HexadecimalString = HighZoneBorder.Background.ToString();
        }

        private void MiddleZone_MouseLeftButtonClick(object sender, MouseButtonEventArgs e)
        {
            HighlightBorder(MiddleZoneBorder);
            ColorIndex = MIDDLE_ZONE_COLOR_INDEX;
            ColorSelect.HexadecimalString = MiddleZoneBorder.Background.ToString();
        }

        private void LowZone_MouseLeftButtonClick(object sender, MouseButtonEventArgs e)
        {
            HighlightBorder(LowZoneBorder);
            ColorIndex = LOW_ZONE_COLOR_INDEX;
            ColorSelect.HexadecimalString = LowZoneBorder.Background.ToString();
        }

        private void Background_MouseLeftButtonClick(object sender, MouseButtonEventArgs e)
        {
            HighlightBorder(BackgroundBorder);
            ColorIndex = BACKGROUND_INDEX;
            ColorSelect.HexadecimalString = BackgroundBorder.Background.ToString();
        }

        private void Frame_MouseLeftButtonClick(object sender, MouseButtonEventArgs e)
        {
            HighlightBorder(FrameBorder);
            ColorIndex = FRAME_COLOR_INDEX;
            ColorSelect.HexadecimalString = FrameBorder.Background.ToString();
        }

        private void SelectedColorChanged(object sender, RoutedPropertyChangedEventArgs<Color?> e)
        {
            switch (ColorIndex)
            {
                case LOW_ZONE_COLOR_INDEX:
                    Matrix_18x14.ColorProfile.LowZone = ColorSelect.HexadecimalString;
                    break;

                case MIDDLE_ZONE_COLOR_INDEX:
                    Matrix_18x14.ColorProfile.MiddleZone = ColorSelect.HexadecimalString;
                    break;

                case HIGH_ZONE_COLOR_INDEX:
                    Matrix_18x14.ColorProfile.HighZone = ColorSelect.HexadecimalString;
                    break;

                case PEAK_COLOR_INDEX:
                    Matrix_18x14.ColorProfile.Peak = ColorSelect.HexadecimalString;
                    break;

                case BACKGROUND_INDEX:
                    LimitColor();
                    Matrix_18x14.ColorProfile.Background = ColorSelect.HexadecimalString;                    
                    break;

                case FRAME_COLOR_INDEX:
                    LimitColor();
                    Matrix_18x14.ColorProfile.Frame = ColorSelect.HexadecimalString;
                    break;
            }

            Matrix_18x14.Update();
        }

        private void ColumnList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            MultiplierBox.DataContext = Matrix_18x14.Columns[ColumnList.SelectedIndex];
            StartFreq.DataContext = Matrix_18x14.Columns[ColumnList.SelectedIndex];
            EndFreq.DataContext = Matrix_18x14.Columns[ColumnList.SelectedIndex];
        }
        
        private void PlusStartMiddleZoneButton_Click(object sender, RoutedEventArgs e)
        {
            Matrix_18x14.Options.MiddleZoneStart++;
        }

        private void MinusStartMiddleZoneButton_Click(object sender, RoutedEventArgs e)
        {
            Matrix_18x14.Options.MiddleZoneStart--;
        }

        private void PlusEndMiddleZoneButton_Click(object sender, RoutedEventArgs e)
        {
            Matrix_18x14.Options.MiddleZoneEnd++;
        }

        private void MinusEndMiddleZoneButton_Click(object sender, RoutedEventArgs e)
        {
            Matrix_18x14.Options.MiddleZoneEnd--;
        }
        
        private void DownloadButton_Click(object sender, RoutedEventArgs e)
        {
            byte[] USBPacket = new byte[USB_PACKET_SIZE];
            USBPacket[0] = USB_REPORT_ID;
            USBPacket[USB_CMD_POS] = USB_CMD_WRITE;
            USBPacket[USB_SUBCMD_POS] = USB_SUBCMD_MATRIX_SET;

            byte[] GRBColor = new byte[3];
            GRBColor = RGBColorStringToGRBArray(Matrix_18x14.ColorProfile.LowZone);
            for (byte ByteNum = 0; ByteNum < MATRIX_LOW_COLOR_SIZE; ByteNum++)
                USBPacket[USB_DATA_POS + MATRIX_LOW_COLOR_OFFSET + ByteNum] = GRBColor[ByteNum];

            GRBColor = RGBColorStringToGRBArray(Matrix_18x14.ColorProfile.MiddleZone);
            for (byte ByteNum = 0; ByteNum < MATRIX_MIDDLE_COLOR_SIZE; ByteNum++)
                USBPacket[USB_DATA_POS + MATRIX_MIDDLE_COLOR_OFFSET + ByteNum] = GRBColor[ByteNum];

            GRBColor = RGBColorStringToGRBArray(Matrix_18x14.ColorProfile.HighZone);
            for (byte ByteNum = 0; ByteNum < MATRIX_HIGH_COLOR_SIZE; ByteNum++)
                USBPacket[USB_DATA_POS + MATRIX_HIGH_COLOR_OFFSET + ByteNum] = GRBColor[ByteNum];

            GRBColor = RGBColorStringToGRBArray(Matrix_18x14.ColorProfile.Peak);
            for (byte ByteNum = 0; ByteNum < MATRIX_PEAK_COLOR_SIZE; ByteNum++)
                USBPacket[USB_DATA_POS + MATRIX_PEAK_COLOR_OFFSET + ByteNum] = GRBColor[ByteNum];

            GRBColor = RGBColorStringToGRBArray(Matrix_18x14.ColorProfile.Background);
            for (byte ByteNum = 0; ByteNum < MATRIX_BACKGROUND_SIZE; ByteNum++)
                USBPacket[USB_DATA_POS + MATRIX_BACKGROUND_OFFSET + ByteNum] = GRBColor[ByteNum];

            GRBColor = RGBColorStringToGRBArray(Matrix_18x14.ColorProfile.Frame);
            for (byte ByteNum = 0; ByteNum < MATRIX_FRAME_COLOR_SIZE; ByteNum++)
                USBPacket[USB_DATA_POS + MATRIX_FRAME_COLOR_OFFSET + ByteNum] = GRBColor[ByteNum];
                        
            USBPacket[USB_DATA_POS + MATRIX_MIDDLE_ZONE_START_OFFSET] = Matrix_18x14.Options.MiddleZoneStart;
            USBPacket[USB_DATA_POS + MATRIX_MIDDLE_ZONE_END_OFFSET] = Matrix_18x14.Options.MiddleZoneEnd;

            USBPacket[USB_DATA_POS + MATRIX_SHOW_OPTIONS_OFFSET] = 0;
            if (Matrix_18x14.Options.IsBackgroundEnabled)
                USBPacket[USB_DATA_POS + MATRIX_SHOW_OPTIONS_OFFSET] |= BACKGROUND_OPTION;
            if (Matrix_18x14.Options.IsFrameEnabled)
                USBPacket[USB_DATA_POS + MATRIX_SHOW_OPTIONS_OFFSET] |= FRAME_OPTION;

            Boolean USBSuccess = USB_device.Write(USBPacket);
            ShowConnectionState(USBSuccess);

            DeviceReadWrite.Operation = USB_CMD_WRITE;
            DeviceReadWrite.Property = USB_SUBCMD_COLUMN_SET;
            DeviceReadWrite.StartColumn = 0;

            ProgressBar.Owner = this;
            ProgressBar.Show();
            ThreadTimer.Start();
        }

        private void ThreadTimer_Tick(object sender, EventArgs e)
        {
            if (DeviceReadWrite.Operation == USB_CMD_WRITE)
            {
                if (DeviceReadWrite.Property == USB_SUBCMD_COLUMN_SET)
                {
                    byte[] USBPacket = new byte[USB_PACKET_SIZE];
                    USBPacket[0] = USB_REPORT_ID;
                    USBPacket[USB_CMD_POS] = USB_CMD_WRITE;
                    USBPacket[USB_SUBCMD_POS] = USB_SUBCMD_COLUMN_SET;
                    USBPacket[USB_COLUMN_POS] = DeviceReadWrite.StartColumn;

                    for (byte ColNum = 0; ColNum < COLUMNS_PER_EEPROM_PAGE; ColNum++)
                    {
                        UInt32 PropertyValue = (UInt32)(Matrix_18x14.Columns[DeviceReadWrite.StartColumn + ColNum].FreqIndexStart + 1);
                        USBPacket[USB_DATA_POS + COLUMN_FREQ_START_OFFSET + ColNum * COLUMN_SETTINGS_SIZE] = (byte)PropertyValue;
                        USBPacket[USB_DATA_POS + COLUMN_FREQ_START_OFFSET + 1 + ColNum * COLUMN_SETTINGS_SIZE] = (byte)(PropertyValue >> 8);

                        PropertyValue = (UInt32)(Matrix_18x14.Columns[DeviceReadWrite.StartColumn + ColNum].FreqIndexEnd + 1);
                        USBPacket[USB_DATA_POS + COLUMN_FREQ_END_OFFSET + ColNum * COLUMN_SETTINGS_SIZE] = (byte)PropertyValue;
                        USBPacket[USB_DATA_POS + COLUMN_FREQ_END_OFFSET + 1 + ColNum * COLUMN_SETTINGS_SIZE] = (byte)(PropertyValue >> 8);

                        PropertyValue = (UInt32)(float.Parse(Matrix_18x14.Columns[DeviceReadWrite.StartColumn + ColNum].Multiplier) * UINT_FLOAT_MULTIPLIER);
                        USBPacket[USB_DATA_POS + COLUMN_MULTIPLIER_OFFSET + ColNum * COLUMN_SETTINGS_SIZE] = (byte)PropertyValue;
                        USBPacket[USB_DATA_POS + COLUMN_MULTIPLIER_OFFSET + 1 + ColNum * COLUMN_SETTINGS_SIZE] = (byte)(PropertyValue >> 8);
                        USBPacket[USB_DATA_POS + COLUMN_MULTIPLIER_OFFSET + 2 + ColNum * COLUMN_SETTINGS_SIZE] = (byte)(PropertyValue >> 16);
                        USBPacket[USB_DATA_POS + COLUMN_MULTIPLIER_OFFSET + 3 + ColNum * COLUMN_SETTINGS_SIZE] = (byte)(PropertyValue >> 24);
                        
                        if ((DeviceReadWrite.StartColumn + ColNum) == (Matrix_18x14.Columns.Count - 1))
                        {
                            ThreadTimer.Stop();
                            ProgressBar.Hide();
                            break;
                        }
                    }

                    DeviceReadWrite.StartColumn += COLUMNS_PER_EEPROM_PAGE;

                    Boolean USBSuccess = USB_device.Write(USBPacket);
                    ShowConnectionState(USBSuccess);
                }
            }

            else if (DeviceReadWrite.Operation == USB_CMD_READ)
            {
                if (DeviceReadWrite.Property == USB_SUBCMD_MATRIX_SET)
                {
                    ProgressBar.Owner = this;
                    ProgressBar.Show();

                    byte[] USBPacket = new byte[USB_PACKET_SIZE];
                    USBPacket[0] = USB_REPORT_ID;
                    USBPacket[USB_CMD_POS] = USB_CMD_READ;
                    USBPacket[USB_SUBCMD_POS] = USB_SUBCMD_MATRIX_SET;

                    Boolean USBSuccess = USB_device.Write(USBPacket);
                    USBSuccess = USB_device.Read(USBPacket);
                    ShowConnectionState(USBSuccess);

                    if (USBPacket[USB_DATA_POS + MATRIX_SHOW_OPTIONS_OFFSET] <= OPTIONS_MASK)
                    {
                        byte[] GRBArray = new byte[MATRIX_LOW_COLOR_SIZE];
                        for (byte ByteNum = 0; ByteNum < MATRIX_LOW_COLOR_SIZE; ByteNum++)
                            GRBArray[ByteNum] = USBPacket[USB_DATA_POS + MATRIX_LOW_COLOR_OFFSET + ByteNum];
                        Matrix_18x14.ColorProfile.LowZone = GRBArrayToRGBColorString(GRBArray);

                        for (byte ByteNum = 0; ByteNum < MATRIX_MIDDLE_COLOR_SIZE; ByteNum++)
                            GRBArray[ByteNum] = USBPacket[USB_DATA_POS + MATRIX_MIDDLE_COLOR_OFFSET + ByteNum];
                        Matrix_18x14.ColorProfile.MiddleZone = GRBArrayToRGBColorString(GRBArray);

                        for (byte ByteNum = 0; ByteNum < MATRIX_HIGH_COLOR_SIZE; ByteNum++)
                            GRBArray[ByteNum] = USBPacket[USB_DATA_POS + MATRIX_HIGH_COLOR_OFFSET + ByteNum];
                        Matrix_18x14.ColorProfile.HighZone = GRBArrayToRGBColorString(GRBArray);

                        for (byte ByteNum = 0; ByteNum < MATRIX_PEAK_COLOR_SIZE; ByteNum++)
                            GRBArray[ByteNum] = USBPacket[USB_DATA_POS + MATRIX_PEAK_COLOR_OFFSET + ByteNum];
                        Matrix_18x14.ColorProfile.Peak = GRBArrayToRGBColorString(GRBArray);

                        for (byte ByteNum = 0; ByteNum < MATRIX_BACKGROUND_SIZE; ByteNum++)
                            GRBArray[ByteNum] = USBPacket[USB_DATA_POS + MATRIX_BACKGROUND_OFFSET + ByteNum];
                        Matrix_18x14.ColorProfile.Background = GRBArrayToRGBColorString(GRBArray);

                        for (byte ByteNum = 0; ByteNum < MATRIX_FRAME_COLOR_SIZE; ByteNum++)
                            GRBArray[ByteNum] = USBPacket[USB_DATA_POS + MATRIX_FRAME_COLOR_OFFSET + ByteNum];
                        Matrix_18x14.ColorProfile.Frame = GRBArrayToRGBColorString(GRBArray);

                        Matrix_18x14.Options.MiddleZoneStart = USBPacket[USB_DATA_POS + MATRIX_MIDDLE_ZONE_START_OFFSET];
                        Matrix_18x14.Options.MiddleZoneEnd = USBPacket[USB_DATA_POS + MATRIX_MIDDLE_ZONE_END_OFFSET];

                        if ((USBPacket[USB_DATA_POS + MATRIX_SHOW_OPTIONS_OFFSET] & BACKGROUND_OPTION) == BACKGROUND_OPTION)
                            Matrix_18x14.Options.IsBackgroundEnabled = true;
                        else
                            Matrix_18x14.Options.IsBackgroundEnabled = false;

                        if ((USBPacket[USB_DATA_POS + MATRIX_SHOW_OPTIONS_OFFSET] & FRAME_OPTION) == FRAME_OPTION)
                            Matrix_18x14.Options.IsFrameEnabled = true;
                        else
                            Matrix_18x14.Options.IsFrameEnabled = false;

                        DeviceReadWrite.Property = USB_SUBCMD_COLUMN_SET;
                        DeviceReadWrite.StartColumn = 0;
                    }

                    else
                    {
                        ThreadTimer.Stop();

                        SetMatrixToDefault();
                        MultiplierBox.DataContext = Matrix_18x14.Columns[0];
                        StartFreq.DataContext = Matrix_18x14.Columns[0];
                        EndFreq.DataContext = Matrix_18x14.Columns[0];

                        ProgressBar.Hide();
                    }
                }

                else if (DeviceReadWrite.Property == USB_SUBCMD_COLUMN_SET)
                {
                    byte[] USBPacket = new byte[USB_PACKET_SIZE];
                    USBPacket[0] = USB_REPORT_ID;
                    USBPacket[USB_CMD_POS] = USB_CMD_READ;
                    USBPacket[USB_SUBCMD_POS] = USB_SUBCMD_COLUMN_SET;
                    USBPacket[USB_COLUMN_POS] = DeviceReadWrite.StartColumn;

                    Boolean USBSuccess = USB_device.Write(USBPacket);
                    USBSuccess = USB_device.Read(USBPacket);
                    ShowConnectionState(USBSuccess);

                    for (byte ColNum = 0; ColNum < COLUMNS_PER_EEPROM_PAGE; ColNum++)
                    {
                        Matrix_18x14.Columns[DeviceReadWrite.StartColumn + ColNum].FreqIndexStart = (UInt16)((USBPacket[USB_DATA_POS + COLUMN_FREQ_START_OFFSET + ColNum * COLUMN_SETTINGS_SIZE] | (USBPacket[USB_DATA_POS + COLUMN_FREQ_START_OFFSET + 1 + ColNum * COLUMN_SETTINGS_SIZE] << 8)) - 1);
                        Matrix_18x14.Columns[DeviceReadWrite.StartColumn + ColNum].FreqIndexEnd = (UInt16)((USBPacket[USB_DATA_POS + COLUMN_FREQ_END_OFFSET + ColNum * COLUMN_SETTINGS_SIZE] | (USBPacket[USB_DATA_POS + COLUMN_FREQ_END_OFFSET + 1 + ColNum * COLUMN_SETTINGS_SIZE] << 8)) - 1);

                        UInt32 MultiplierValue = 0;
                        for (byte ByteNum = 0; ByteNum < COLUMN_MULTIPLIER_SIZE; ByteNum++)
                            MultiplierValue |= (UInt32)(USBPacket[USB_DATA_POS + COLUMN_MULTIPLIER_OFFSET + ByteNum + ColNum * COLUMN_SETTINGS_SIZE] << (ByteNum * 8));
                        
                        Matrix_18x14.Columns[DeviceReadWrite.StartColumn + ColNum].Multiplier = ((float)(MultiplierValue) / UINT_FLOAT_MULTIPLIER).ToString();
                        
                        if ((DeviceReadWrite.StartColumn + ColNum) == (Matrix_18x14.Columns.Count - 1))
                        {
                            ThreadTimer.Stop();
                            ProgressBar.Hide();

                            MultiplierBox.DataContext = Matrix_18x14.Columns[0];
                            StartFreq.DataContext = Matrix_18x14.Columns[0];
                            EndFreq.DataContext = Matrix_18x14.Columns[0];

                            break;
                        }
                    }

                    DeviceReadWrite.StartColumn += COLUMNS_PER_EEPROM_PAGE;
                }
            }
        }

        private byte[] RGBColorStringToGRBArray(string RGBColorString)
        {
            UInt32 ColorFromString = UInt32.Parse(RGBColorString.Substring(1), NumberStyles.HexNumber);
            byte[] GRBArray = new byte[MATRIX_LOW_COLOR_SIZE];
            
            GRBArray[2] = (byte)((ColorFromString) >> 8);
            GRBArray[0] = (byte)(ColorFromString);
            GRBArray[1] = (byte)((ColorFromString) >> 16);
            
            return GRBArray;
        }

        private string GRBArrayToRGBColorString(byte[] GRBArray)
        {
            UInt32 IntColor = (UInt32)(GRBArray[0] | (GRBArray[2] << 8) | (GRBArray[1] << 16));
            string StringColor;
            StringColor = "#" + Convert.ToString(IntColor, 16);
            while (StringColor.Length < ColorStringLength)
                StringColor = StringColor.Insert(1, "0");

            return StringColor;
        }

        private void LimitColor()
        {
            bool IsWarningPending = false;

            if (ColorSelect.R > COLOR_COMPONENT_MAX_VALUE)
            {
                ColorSelect.R = COLOR_COMPONENT_MAX_VALUE;
                IsWarningPending = true;
            }
                

            if (ColorSelect.G > COLOR_COMPONENT_MAX_VALUE)
            {
                ColorSelect.G = COLOR_COMPONENT_MAX_VALUE;
                IsWarningPending = true;
            }
                

            if (ColorSelect.B > COLOR_COMPONENT_MAX_VALUE)
            {
                ColorSelect.B = COLOR_COMPONENT_MAX_VALUE;
                IsWarningPending = true;
            }
                
            if (IsWarningPending)
            {
                MessageBox.Show(UserInputErrorColor, UserInputErrorHeader, MessageBoxButton.OK, MessageBoxImage.Warning);
            }
        }

        private void SetMatrixToDefault()
        {
            MatrixColors UserColors = new MatrixColors();
            UserColors.LowZone = LowZoneColorDef;
            UserColors.MiddleZone = MiddleZoneColorDef;
            UserColors.HighZone = HighZoneColorDef;
            UserColors.Peak = PeakColorDef;
            UserColors.Background = BackgroundDef;
            UserColors.Frame = FrameColDef;
            Matrix_18x14.SetColorProfile(UserColors);

            Matrix_18x14.Options.MiddleZoneStart = MiddleZoneStartDefault;
            Matrix_18x14.Options.MiddleZoneEnd = MiddleZoneEndDefault;
            Matrix_18x14.Options.IsBackgroundEnabled = false;
            Matrix_18x14.Options.IsFrameEnabled = false;
            
            for (byte ColNum = 0; ColNum < Matrix_18x14.Columns.Count; ColNum++)
            {
                Matrix_18x14.Columns[ColNum].FreqIndexStart = (UInt16)(DefaultBandsStart[ColNum] - 1);
                Matrix_18x14.Columns[ColNum].FreqIndexEnd = (UInt16)(DefaultBandsEnd[ColNum] - 1);
                Matrix_18x14.Columns[ColNum].Multiplier = DefaultMultipliers[ColNum].ToString();
            }
        }

        private void WindowIsClosing(object sender, CancelEventArgs e)
        {
            ProgressBar.Close();
            USB_device.Close();
        }

        private void ShowConnectionState(bool State)
        {
            if (State)
            {
                StateImage.Source = new BitmapImage(new Uri("images/apply_32x32.png", UriKind.Relative));
                StateTextBlock.Text = DeviceConnectedString;
            }

            else
            {
                StateImage.Source = new BitmapImage(new Uri("images/warning_32x32.png", UriKind.Relative));
                StateTextBlock.Text = DeviceNotConnectedString;
                FirmwareTextBlock.Text = " ";
            }
        }

        private void OpenFileButton_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog ChooseFile = new OpenFileDialog();
            ChooseFile.Filter = "Text files (*.txt)|*.txt";
            if (ChooseFile.ShowDialog() == true)
            {
                OpenFile(ChooseFile.FileName);
            }
        }

        private void OpenFile(string FileToOpen)
        {
            using (StreamReader FileToReadFrom = File.OpenText(FileToOpen))
            {                
                if (FileToReadFrom.Peek() >= 0)
                {
                    FileToReadFrom.ReadLine();
                    StringBuilder NewLine;

                    for (byte PropNum = 0; PropNum < MatrixColorsCount; PropNum++)
                    {                        
                        if (!(FileToReadFrom.Peek() >= 0))
                            break;

                        NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                        NewLine.Replace(" ", "");

                        string PropString = NewLine.ToString().Substring(0, NewLine.ToString().IndexOf("="));
                        string PropValueString = NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1);

                        switch (Array.IndexOf(ColorsFileStrings, PropString))
                        {
                            case LOW_ZONE_COLOR_INDEX:
                                Matrix_18x14.ColorProfile.LowZone = PropValueString;
                                break;

                            case MIDDLE_ZONE_COLOR_INDEX:
                                Matrix_18x14.ColorProfile.MiddleZone = PropValueString;
                                break;

                            case HIGH_ZONE_COLOR_INDEX:
                                Matrix_18x14.ColorProfile.HighZone = PropValueString;
                                break;

                            case PEAK_COLOR_INDEX:
                                Matrix_18x14.ColorProfile.Peak = PropValueString;
                                break;

                            case BACKGROUND_INDEX:
                                LimitColor();
                                Matrix_18x14.ColorProfile.Background = PropValueString;
                                break;

                            case FRAME_COLOR_INDEX:
                                LimitColor();
                                Matrix_18x14.ColorProfile.Frame = PropValueString;
                                break;
                        }
                    }

                    if (!(FileToReadFrom.Peek() >= 0))
                        return;
                    NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                    NewLine.Replace(" ", "");
                    Matrix_18x14.Options.MiddleZoneStart = byte.Parse(NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1));

                    if (!(FileToReadFrom.Peek() >= 0))
                        return;
                    NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                    NewLine.Replace(" ", "");
                    Matrix_18x14.Options.MiddleZoneEnd = byte.Parse(NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1));

                    if (!(FileToReadFrom.Peek() >= 0))
                        return;
                    NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                    NewLine.Replace(" ", "");
                    Matrix_18x14.Options.IsBackgroundEnabled = bool.Parse(NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1));

                    if (!(FileToReadFrom.Peek() >= 0))
                        return;
                    NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                    NewLine.Replace(" ", "");
                    Matrix_18x14.Options.IsFrameEnabled = bool.Parse(NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1));
                    
                    if (FileToReadFrom.Peek() >= 0)                    
                        FileToReadFrom.ReadLine();

                    for (byte i = 0; i < Matrix_18x14.Columns.Count; i++)
                    {
                        if (!(FileToReadFrom.Peek() >= 0))
                            break;

                        NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                        NewLine.Replace(" ", "");

                        int StartIndex = NewLine.ToString().IndexOf("[");
                        int EndIndex = NewLine.ToString().IndexOf("]");

                        int ColNum = 0;
                        if (int.TryParse(NewLine.ToString().Substring(StartIndex + 1, EndIndex - StartIndex - 1), out ColNum) == false)
                            break;

                        ColNum--;

                        if (!(FileToReadFrom.Peek() >= 0))
                            break;
                        NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                        NewLine.Replace(" ", "");
                        Matrix_18x14.Columns[ColNum].FreqIndexStart = UInt16.Parse(NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1));

                        if (!(FileToReadFrom.Peek() >= 0))
                            break;
                        NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                        NewLine.Replace(" ", "");
                        Matrix_18x14.Columns[ColNum].FreqIndexEnd = UInt16.Parse(NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1));

                        if (!(FileToReadFrom.Peek() >= 0))
                            break;
                        NewLine = new StringBuilder(FileToReadFrom.ReadLine());
                        NewLine.Replace(" ", "");
                        Matrix_18x14.Columns[ColNum].Multiplier = NewLine.ToString().Remove(0, NewLine.ToString().IndexOf("=") + 1);                        
                    }
                }
            }

            Matrix_18x14.Update();
        }

        private void SaveFileButton_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog ChooseFile = new SaveFileDialog();
            ChooseFile.Filter = "Text files (*.txt)|*.txt";

            if (ChooseFile.ShowDialog() == true)
            {
                using (StreamWriter FileToWriteTo = File.CreateText(ChooseFile.FileName))
                {
                    FileToWriteTo.WriteLine("--------- {0} ---------", MatrixSettingsFileString);
                    FileToWriteTo.WriteLine("{0} = {1}", LowZoneFileString, Matrix_18x14.ColorProfile.LowZone);
                    FileToWriteTo.WriteLine("{0} = {1}", MiddleZoneFileString, Matrix_18x14.ColorProfile.MiddleZone);
                    FileToWriteTo.WriteLine("{0} = {1}", HighZoneFileString, Matrix_18x14.ColorProfile.HighZone);
                    FileToWriteTo.WriteLine("{0} = {1}", PeakFileString, Matrix_18x14.ColorProfile.Peak);
                    FileToWriteTo.WriteLine("{0} = {1}", BackgroundFileString, Matrix_18x14.ColorProfile.Background);
                    FileToWriteTo.WriteLine("{0} = {1}", FrameFileString, Matrix_18x14.ColorProfile.Frame);
                    FileToWriteTo.WriteLine("{0} = {1}", MiddleZoneStartFileString, Matrix_18x14.Options.MiddleZoneStart);
                    FileToWriteTo.WriteLine("{0} = {1}", MiddleZoneEndFileString, Matrix_18x14.Options.MiddleZoneEnd);
                    FileToWriteTo.WriteLine("{0} = {1}", IsBackgroundFileString, Matrix_18x14.Options.IsBackgroundEnabled);
                    FileToWriteTo.WriteLine("{0} = {1}", IsFrameFileString, Matrix_18x14.Options.IsFrameEnabled);

                    FileToWriteTo.WriteLine("--------- {0} ---------", ColumnsSettingsFileString);
                    
                    for (byte ColNum = 0; ColNum < Matrix_18x14.Columns.Count; ColNum++)
                    {
                        FileToWriteTo.WriteLine("--------- Column[{0}] ---------", ColNum + 1);
                        FileToWriteTo.WriteLine("{0} = {1}", ColumnsFreqStartFileString, Matrix_18x14.Columns[ColNum].FreqIndexStart);
                        FileToWriteTo.WriteLine("{0} = {1}", ColumnsFreqEndFileString, Matrix_18x14.Columns[ColNum].FreqIndexEnd);
                        FileToWriteTo.WriteLine("{0} = {1}", ColumnsMultiplierFileString, Matrix_18x14.Columns[ColNum].Multiplier);
                    }
                }
            }
        }

        private void HighlightBorder(Object SelectedObject)
        {
            PeakBorder.BorderThickness = new Thickness(DeSelectThickness);
            HighZoneBorder.BorderThickness = new Thickness(DeSelectThickness);
            MiddleZoneBorder.BorderThickness = new Thickness(DeSelectThickness);
            LowZoneBorder.BorderThickness = new Thickness(DeSelectThickness);
            BackgroundBorder.BorderThickness = new Thickness(DeSelectThickness);
            FrameBorder.BorderThickness = new Thickness(DeSelectThickness);

            (SelectedObject as Border).BorderThickness = new Thickness(SelectThickness);
        }
    }
}


