﻿/*
********************************************************************************
* COPYRIGHT(c) ЗАО «ЧИП и ДИП», 2019
* 
* Программное обеспечение предоставляется на условиях «как есть» (as is).
* При распространении указание автора обязательно.
********************************************************************************
*/



using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace SpectrA
{
    public class MatrixOptions : INotifyPropertyChanged
    {
        private const byte MiddleZoneStartMin = 1;
        private const byte MiddleZoneStartMax = 8;
        private const byte MiddleZoneEndMin = 5;
        private const byte MiddleZoneEndMax = 10;

        private byte middlezonestart;
        private byte middlezoneend;
        private bool isbackgroundenabled = false;
        private bool isframeenabled = false;


        public byte MiddleZoneStart
        {
            get { return this.middlezonestart; }
            set
            {
                if (this.middlezonestart != value)
                {
                    if (value < MiddleZoneStartMin)
                        value = MiddleZoneStartMin;
                    else if (value > MiddleZoneStartMax)
                        value = MiddleZoneStartMax;

                    this.middlezonestart = value;

                    this.NotifyPropertyChanged("MiddleZoneStart");

                    OptionChanged?.Invoke();
                }
            }
        }

        public byte MiddleZoneEnd
        {
            get { return this.middlezoneend; }
            set
            {
                if (this.middlezoneend != value)
                {
                    if (value < MiddleZoneEndMin)
                        value = MiddleZoneEndMin;
                    else if (value > MiddleZoneEndMax)
                        value = MiddleZoneEndMax;

                    this.middlezoneend = value;
                    
                    this.NotifyPropertyChanged("MiddleZoneEnd");

                    OptionChanged?.Invoke();
                }
            }
        }

        public bool IsBackgroundEnabled
        {
            get { return this.isbackgroundenabled; }
            set
            {
                if (this.isbackgroundenabled != value)
                {
                    this.isbackgroundenabled = value;

                    this.NotifyPropertyChanged("IsBackgroundEnabled");

                    OptionChanged?.Invoke();
                }
            }
        }

        public bool IsFrameEnabled
        {
            get { return this.isframeenabled; }
            set
            {
                if (this.isframeenabled != value)
                {
                    this.isframeenabled = value;

                    this.NotifyPropertyChanged("IsFrameEnabled");

                    OptionChanged?.Invoke();
                }
            }
        }
        
        public event PropertyChangedEventHandler PropertyChanged;
        public void NotifyPropertyChanged(string propName)
        {
            this.PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propName));
        }

        public delegate void OptionChangedEventHandler();
        public event OptionChangedEventHandler OptionChanged;
    }
}
