﻿/*
********************************************************************************
* COPYRIGHT(c) ЗАО «ЧИП и ДИП», 2019
* 
* Программное обеспечение предоставляется на условиях «как есть» (as is).
* При распространении указание автора обязательно.
********************************************************************************
*/


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SpectrA
{
    public class PixelMatrix
    {
        public List<Column> Columns { get; set; }
        public MatrixColors ColorProfile { get; set; }
        public MatrixOptions Options { get; set; }
        
        public PixelMatrix(byte ColumnsCount, byte RowsCount)
        {
            ColorProfile = new MatrixColors();
            Columns = new List<Column>();
            Options = new MatrixOptions();
            Options.OptionChanged += new MatrixOptions.OptionChangedEventHandler(Update);

            for (byte i = 0; i < ColumnsCount; i++)
            {
                Column NewColumn = new Column(RowsCount);
                NewColumn.LastActivePixel = (byte)(RowsCount - 3);
                NewColumn.PeakPixel = (byte)(RowsCount - 2);
                Columns.Add(NewColumn);
            }            
        }

        public void SetColorProfile(MatrixColors NewColors)
        {
            ColorProfile.LowZone = NewColors.LowZone;
            ColorProfile.MiddleZone = NewColors.MiddleZone;
            ColorProfile.HighZone = NewColors.HighZone;
            ColorProfile.Peak = NewColors.Peak;
            ColorProfile.Background = NewColors.Background;
            ColorProfile.Frame = NewColors.Frame;
        }

        public void SetOptions(MatrixOptions NewOptions)
        {
            Options.MiddleZoneStart = NewOptions.MiddleZoneStart;
            Options.MiddleZoneEnd = NewOptions.MiddleZoneEnd;
            Options.IsBackgroundEnabled = NewOptions.IsBackgroundEnabled;
            Options.IsFrameEnabled = NewOptions.IsFrameEnabled;
        }

        public void Update()
        {
            foreach (Column Item in Columns)
                Item.Update(ColorProfile, Options);

            if (Options.IsFrameEnabled == true)
                ShowFrame();
        }

        private void ShowFrame()
        {
            for (byte PixelNum = 0; PixelNum < Columns[0].Pixels.Count; PixelNum++)
            {
                Columns[0].Pixels[PixelNum].Color = ColorProfile.Frame;
                Columns[Columns.Count - 1].Pixels[PixelNum].Color = ColorProfile.Frame;
            }
            
            foreach(Column Item in Columns)
            {
                Item.Pixels[0].Color = ColorProfile.Frame;
                Item.Pixels[Item.Pixels.Count - 1].Color = ColorProfile.Frame;
            }
        }
    }
}
